%run me
d = 20;                                 %m, focusing distance
plotFlag = true;

N = 2.^(2:0.5/3:4);
ns = 20;                                %distance samples in decade
u2 = (10.^(1/ns)).^(-30:61);            %m
th = 50;                                %lp/mm, MTF50 'acceptable' sharpness

ln = length(N);
mMax = zeros(1,ln);
mMaxu = zeros(1,ln);
un = zeros(1,ln);
uf = zeros(1,ln);
m1k = zeros(1,ln);
un1k = zeros(1,ln);
mtf50s = zeros(ln,length(u2));

for i = 1:ln
    [u2,mtf50] = DOFMTF50(24,N(i),d,false,u2,18.3);
    mtf50 = mtf50*1000;                 %lp/mm
    
    [mMax(i),mMaxu(i),un(i),uf(i),m1k(i),un1k(i)] = dofStats(u2,mtf50,th);
    
    mtf50s(i,:) = mtf50;
end

[~,I] = max(mMax);
fprintf(['Best In-focus   f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
    ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(I),m1k(I),uf(I),un(I),un1k(I));
[m1kMax,I] = max(m1k);
fprintf(['Best Vanish     f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
    ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(I),m1k(I),uf(I),un(I),un1k(I));
[unMin,I] = min(un);
fprintf(['Best Near Limit f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
    ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(I),m1k(I),uf(I),un(I),un1k(I));

sm = (mMax(3:end) - mMax(1:end-2))/2;
s1k = (m1k(3:end) - m1k(1:end-2))/2;
I = find((sm+s1k)>=0,1,'last') + 1;
fprintf(['Best Compromise f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
    ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(I),m1k(I),uf(I),un(I),un1k(I));

if plotFlag
    figure
    plot(N,uf,'.-k',N,un,'k',N,un1k,'--k')
    set(gca,'xscale','log'); grid
    xlabel('f-number'), ylabel('Distance to Threshold (m)')
    xticks(N); xticklabels(split(num2str(N,'%5.1f')))
    yticks([0.1:0.1:0.5 0.7 1:5 7 10 15 20 30 40 60 100 150 200 300 400 600 1000])
    set(gca,'yscale','log'); ylim([1 1000])
    
    yyaxis right
    plot(N,mMax,'-','color',[0 0.4470 0.7410]); hold on
    plot(N,m1k,'--','color',[0 0.4470 0.7410])
    plot(N,th*ones(1,length(N)),'-','color',[0 0.4470 0.7410],'LineWidth',.2)
    ylim([0 100]); ylabel('MTF50 (lp/mm)')
    legend(['Far   ' num2str(th) 'lp/mm Limit'],['Near ' num2str(th) 'lp/mm Limit'],...
        'Near same as 1k','In-focus MTF50','1Km MTF50')
    title(['Z7+24-70mm/4 focused at ' num2str(d) 'm, '...
        'Threshold MTF50 = ' num2str(th) ' lp/mm'])
    yyaxis left; hold off
end
